#!/usr/bin/python3
#
# examples/xfixes-cursor-notify.py -- demonstrate the XFIXES extension
# CursorNotify event.
#
#    Copyright (C) 2022
#      Dan Isla <dan.isla@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
from Xlib.display import Display
from Xlib.ext import xfixes

def main():
    display = Display()

    if not display.has_extension('XFIXES'):
        if display.query_extension('XFIXES') is None:
            print('XFIXES extension not supported')
            return 1

    xfixes_version = display.xfixes_query_version()
    print('Found XFIXES version {}.{}'.format(
        xfixes_version.major_version,
        xfixes_version.minor_version
    ))

    screen = display.screen()

    display.xfixes_select_cursor_input(screen.root, xfixes.XFixesDisplayCursorNotifyMask)

    cursor_cache = {}
  
    while True:
        e = display.next_event()
        print(e)

        if (e.type, e.sub_code) == display.extension_event.DisplayCursorNotify:
            print("DisplayCursorNotify: cursor_serial={}".format(e.cursor_serial))
            image = display.xfixes_get_cursor_image(screen.root)
            cached = False
            if cursor_cache.get(image.cursor_serial):
                cached = True
            else:
                cursor_cache[image.cursor_serial] = image.cursor_image

            print("Cursor position={},{}, size={}x{}, xyhot={},{}, cursor_serial={}, cached={}".format(
                image.x, image.y, image.width,image.height, image.xhot, image.yhot, image.cursor_serial, cached
            ))


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python
#
# examples/childwin.py -- demonstrate child windows.
#
#	Copyright (C) 2008 David Bronke <whitelynx@gmail.com>
#	Copyright (C) 2002 Peter Liljenberg <petli@ctrl-c.liu.se>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib import X, display, Xutil

# Application window
class Window(object):
	def __init__(self, display):
		self.d = display

		# Find which screen to open the window on
		self.screen = self.d.screen()

		# background pattern
		bgsize = 20

		bgpm = self.screen.root.create_pixmap(
			bgsize,
			bgsize,
			self.screen.root_depth
			)

		bggc = self.screen.root.create_gc(
			foreground=self.screen.black_pixel,
			background=self.screen.black_pixel
			)

		bgpm.fill_rectangle(bggc, 0, 0, bgsize, bgsize)

		bggc.change(foreground=self.screen.white_pixel)

		bgpm.arc(bggc, -bgsize // 2, 0, bgsize, bgsize, 0, 360 * 64)
		bgpm.arc(bggc, bgsize // 2, 0, bgsize, bgsize, 0, 360 * 64)
		bgpm.arc(bggc, 0, -bgsize // 2, bgsize, bgsize, 0, 360 * 64)
		bgpm.arc(bggc, 0, bgsize // 2, bgsize, bgsize, 0, 360 * 64)

		# Actual window
		self.window = self.screen.root.create_window(
			100, 100, 400, 300, 0,
			self.screen.root_depth,
			X.InputOutput,
			X.CopyFromParent,

			# special attribute values
			background_pixmap=bgpm,
			event_mask=(
				X.StructureNotifyMask |
				X.ButtonReleaseMask
				),
			colormap=X.CopyFromParent
			)

		# Set some WM info

		self.WM_DELETE_WINDOW = self.d.intern_atom('WM_DELETE_WINDOW')
		self.WM_PROTOCOLS = self.d.intern_atom('WM_PROTOCOLS')

		self.window.set_wm_name('Xlib example: childwin.py')
		self.window.set_wm_icon_name('childwin.py')
		self.window.set_wm_class('childwin', 'XlibExample')

		self.window.set_wm_protocols([self.WM_DELETE_WINDOW])
		self.window.set_wm_hints(
			flags=Xutil.StateHint,
			initial_state=Xutil.NormalState
			)

		self.window.set_wm_normal_hints(
			flags=(Xutil.PPosition | Xutil.PSize | Xutil.PMinSize),
			min_width=50,
			min_height=50
			)

		# Map the window, making it visible
		self.window.map()

		# Child window
		(self.childWidth, self.childHeight) = (20, 20)
		self.childWindow = self.window.create_window(
			20, 20, self.childWidth, self.childHeight, 0,
			self.screen.root_depth,
			X.CopyFromParent,
			X.CopyFromParent,

			# special attribute values
			background_pixel=self.screen.white_pixel,
			colormap=X.CopyFromParent,
			)
		self.childWindow.map()


	# Main loop, handling events
	def loop(self):
		current = None
		while 1:
			e = self.d.next_event()

			# Window has been destroyed, quit
			if e.type == X.DestroyNotify:
				sys.exit(0)

			# Button released, add or subtract
			elif e.type == X.ButtonRelease:
				if e.detail == 1:
					print("Moving child window.")
					self.childWindow.configure(
						x=e.event_x - self.childWidth // 2,
						y=e.event_y - self.childHeight // 2
						)
					self.d.flush()

			# Somebody wants to tell us something
			elif e.type == X.ClientMessage:
				if e.client_type == self.WM_PROTOCOLS:
					fmt, data = e.data
					if fmt == 32 and data[0] == self.WM_DELETE_WINDOW:
						sys.exit(0)


if __name__ == '__main__':
	Window(display.Display()).loop()




import unittest

# Python 2/3 compatibility.
from six import indexbytes, text_type

from Xlib.protocol.display import bytesview


class BytesViewTest(unittest.TestCase):

    def test(self):
        with self.assertRaises(TypeError):
            bytesview(text_type('foobar'))
        data = b'0123456789ABCDEF'
        view = bytesview(data)
        self.assertEqual(len(view), 16)
        self.assertEqual(view[:], data)
        self.assertIsInstance(view[:], bytes)
        self.assertEqual(view[5:-6], b'56789')
        self.assertEqual(indexbytes(view, 7), ord('7'))
        view = bytesview(view, 5)
        self.assertEqual(view[:], b'56789ABCDEF')
        self.assertEqual(indexbytes(view, 4), ord('9'))
        view = bytesview(view, 0, 5)
        self.assertEqual(view[:], b'56789')
        self.assertEqual(indexbytes(view, 1), ord('6'))



#!/usr/bin/env python

import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import Xlib.display
import Xlib.error
import Xlib.protocol.display
import Xlib.protocol.event
import Xlib.protocol.rq
import Xlib.xobject.fontable
import Xlib.X


class TestXlibDisplay(unittest.TestCase):
    def setUp(self):
        # Create authority file.
        self.display_num = os.getenv("DISPLAY")
        self.display = Xlib.display.Display(self.display_num)
        self.dummy_str = "qqq"
        self.keysym = 65535

    def test_display_instance(self):
        self.assertTrue(isinstance(self.display, Xlib.display.Display))

    def test_default_display_name(self):
        self.assertEqual(self.display.get_display_name(), self.display_num)

    def test_default_screen_number(self):
        self.assertEqual(self.display.get_default_screen(), 0)

    def test_returns_no_events(self):
        self.assertEqual(self.display.pending_events(), 0)

    def test_pointer_mapping_is_list(self):
        self.assertTrue(isinstance(self.display.get_pointer_mapping(), list))

    def test_set_get_pointer_mapping(self):
        orig_mapping = self.display.get_pointer_mapping()
        length = len(orig_mapping)
        try:
            self.display.set_pointer_mapping([0] * length)
            self.assertEqual(self.display.get_pointer_mapping(), [0] * length)
        finally:
            self.display.set_pointer_mapping(orig_mapping)

    def test_can_close_display(self):
        self.display.close()
        self.assertEqual(str(self.display.display.socket_error), "Display connection closed by client")

    def test_can_close_display_and_check_for_error(self):
        self.display.close()
        self.assertRaises(Xlib.error.ConnectionClosedError, self.display.flush)

    def test_return_fileno(self):
        self.assertTrue(isinstance(self.display.fileno(), int))

    def test_has_no_invalid_extension(self):
        self.assertTrue(~self.display.has_extension(self.dummy_str))

    def test_has_valid_extension(self):
        extensions = self.display.list_extensions()
        if extensions:
            self.assertTrue(~self.display.has_extension(extensions[0]))

    def test_can_create_resource_object(self):
        self.assertTrue(
            isinstance(self.display.create_resource_object("font", 0), Xlib.xobject.fontable.Font))

    def test_get_default_screen_instance(self):
        self.assertTrue(isinstance(self.display.screen(), Xlib.protocol.rq.DictWrapper))

    def test_get_zero_screen_instance(self):
        self.assertTrue(isinstance(self.display.screen(0), Xlib.protocol.rq.DictWrapper))

    def test_default_screen_count(self):
        self.assertEqual(self.display.screen_count(), 1)

    def test_cannot_add_existing_display_method(self):
        self.assertRaises(AssertionError, self.display.extension_add_method,
                          "display", "extension_add_method", lambda x: x)

    def test_cannot_add_existing_font_method(self):
        self.assertRaises(AssertionError, self.display.extension_add_method, "font", "__init__", lambda x: x)

    def test_can_add_extension_error(self):
        self.display.extension_add_error(1, Xlib.error.XError)
        self.assertEqual(self.display.display.error_classes[1], Xlib.error.XError)

    def test_keycode_to_keysym_for_invalid_index(self):
        self.assertEqual(self.display.keycode_to_keysym(0, 0), Xlib.X.NoSymbol)

    def test_keysym_to_keycode_for_nosymbol(self):
        self.assertEqual(self.display.keysym_to_keycode(Xlib.X.NoSymbol), 0)

    def test_keysym_to_keycode_for_valid_symbol(self):
        self.assertEqual(self.display.keysym_to_keycode(self.keysym), 119)

    def test_keysym_to_keycodes_for_nosymbol(self):
        self.assertEqual(self.display.keysym_to_keycodes(Xlib.X.NoSymbol), [])

    def test_refresh_keyboard_mapping_invalid_event(self):
        self.assertRaises(TypeError, self.display.refresh_keyboard_mapping, Xlib.protocol.event.AnyEvent)

    def test_get_modifier_mapping(self):
        self.assertEqual(len(self.display.get_modifier_mapping()), 8)

    def test_set_modifier_mapping(self):
        mapping = self.display.get_modifier_mapping()
        self.assertEqual(self.display.set_modifier_mapping(mapping), Xlib.X.MappingSuccess)

    def test_get_screensaver(self):
        self.assertTrue(isinstance(self.display.get_screen_saver(), Xlib.protocol.request.GetScreenSaver))

    def test_list_hosts(self):
        self.assertTrue(isinstance(self.display.list_hosts(), Xlib.protocol.request.ListHosts))

    def test_get_keyboard_control(self):
        self.assertTrue(
            isinstance(self.display.get_keyboard_control(), Xlib.protocol.request.GetKeyboardControl))

    def test_change_keyboard_mapping(self):
        kpt_mapping = self.display.get_keyboard_mapping(254, 1)
        self.display.change_keyboard_mapping(254, kpt_mapping)
        self.assertEqual(self.display.get_keyboard_mapping(254, 1), kpt_mapping)

    def test_get_font_path(self):
        self.assertNotEqual(self.display.get_font_path(), [])

    def test_get_atom_name(self):
        atom = self.display.get_atom(self.display_num)
        val = self.display.get_atom_name(atom)
        self.assertEqual(val, self.display_num)

    def test_intern_atom(self):
        atom = self.display.intern_atom(self.display_num)
        val = self.display.get_atom_name(atom)
        self.assertEqual(val, self.display_num)

    def test_get_input_focus(self):
        self.assertTrue(isinstance(self.display.get_input_focus(), Xlib.protocol.request.GetInputFocus))

    def test_query_keymap(self):
        self.assertTrue(isinstance(self.display.query_keymap(), list))

    def test_open_invalid_font(self):
        self.assertEqual(self.display.open_font(self.dummy_str), None)

    def test_list_fonts(self):
        fonts = self.display.list_fonts("*", 1)
        self.assertNotEqual(fonts, [])

    def test_lookup_valid_keysym(self):
        self.assertNotEqual(self.display.lookup_string(self.keysym), None)

    def test_lookup_invalid_keysym(self):
        self.assertEqual(self.display.lookup_string(-1), None)

    def test_rebind_string(self):
        self.display.rebind_string(self.keysym, self.dummy_str)
        self.assertEqual(self.display.lookup_string(self.keysym), self.dummy_str)

    def test_get_selection_owner(self):
        atom = self.display.get_atom(self.display_num)
        self.assertEqual(self.display.get_selection_owner(atom), 0)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
#
# examples/xdamage.py -- demonstrate damage extension
#
#    Copyright (C) 2019 Mohit Garg <mrmohitgarg1990@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA


# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib import display, X, threaded,Xutil
import time

try:
    import thread
except ModuleNotFoundError:
    import _thread as thread

from Xlib.ext import damage
from PIL import Image, ImageTk
import traceback

def redraw(win, gc):
    # win.clear_area()
    win.fill_rectangle(gc, 0, 0, 60, 60)

def blink(display, win, gc, cols):
    while 1:
        time.sleep(2)
        print('Changing color', cols[0])
        gc.change(foreground = cols[0])
        cols = (cols[1], cols[0])
        redraw(win, gc)
        display.flush()

def get_image_from_win(win, pt_w, pt_h, pt_x=0, pt_y=0):
    try:
        raw = win.get_image(pt_x, pt_y, pt_w, pt_h, X.ZPixmap, 0xffffffff)
        image = Image.frombytes("RGB", (pt_w, pt_h), raw.data, "raw", "BGRX")
        return image

    except Exception:
        traceback.print_exc()

def check_ext(disp):
    # Check for extension
    if not disp.has_extension('DAMAGE'):
        sys.stderr.write('server does not have the DAMAGE extension\n')
        sys.stderr.write("\n".join(disp.list_extensions()))
        
        if disp.query_extension('DAMAGE') is None:
            sys.exit(1)
    else:
        r = disp.damage_query_version()
        print('DAMAGE version {}.{}'.format(r.major_version, r.minor_version))
    
def main():
    d = display.Display()
    root = d.screen().root

    check_ext(d)
    colormap = d.screen().default_colormap

    red = colormap.alloc_named_color("red").pixel
    blue = colormap.alloc_named_color("blue").pixel
    background = colormap.alloc_named_color("white").pixel

    window1 = root.create_window(100, 100, 250, 100, 1,
                                X.CopyFromParent, X.InputOutput,
                                X.CopyFromParent,
                                background_pixel = background,
                                event_mask = X.StructureNotifyMask | X.ExposureMask)
    window1.set_wm_name('Changing Window')
    window1.map()
    gc = window1.create_gc(foreground = red)
    
    thread.start_new_thread(blink, (d, window1, gc, (blue, red)))
    
    window1.damage_create(damage.DamageReportRawRectangles)
    window1.set_wm_normal_hints(
            flags=(Xutil.PPosition | Xutil.PSize | Xutil.PMinSize),
            min_width=50,
            min_height=50
            )

    window2 = root.create_window(100, 250, 250, 100, 1,
                                X.CopyFromParent, X.InputOutput,
                                X.CopyFromParent,
                                background_pixel = background,
                                event_mask = X.StructureNotifyMask | X.ExposureMask)
    window2.set_wm_normal_hints(
            flags=(Xutil.PPosition | Xutil.PSize | Xutil.PMinSize),
            min_width=50,
            min_height=50
            )

    window2.set_wm_name('Tracking Window')
    window2.map()

    while 1:
        event = d.next_event()
        if event.type == X.Expose:
            if event.count == 0:
                redraw(window1, gc)
        elif event.type == d.extension_event.DamageNotify:
            image = get_image_from_win(window1, event.area.width, event.area.height, event.area.x, event.area.y)
            bgpm = window2.create_pixmap(image.width, image.height, d.screen().root_depth)
            bggc = window2.create_gc(foreground=0, background=0)
            bgpm.put_pil_image(bggc, 0, 0, image)
            window2.copy_area(bggc, bgpm, 0, 0, image.width, image.height, 0, 0)
            # bggc.free()
        elif event.type == X.DestroyNotify:
            sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# examples/dpms.py -- DPMS usage examples.
#
#    Copyright (C) 2020 Thiago Kenji Okada <thiagokokada@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

import random
import time

from Xlib import display
from Xlib.ext import dpms


class DPMSExamples(object):
    def __init__(self):
        self.d = display.Display()

        capable = self.d.dpms_capable()
        assert capable, 'DPMS is not supported in your system'

        self.initial_info = self.d.dpms_info()
        self.initial_timeouts = self.d.dpms_get_timeouts()

        # Making sure that DPMS is enable for this examples
        self.d.dpms_enable()
        self.d.sync()

    def print_dpms(self):
        current_info = self.d.dpms_info()
        print('\nDPMS state: {}\nPower level: {}'.format(current_info.state,
                                                         current_info.power_level))
        current_timeouts = self.d.dpms_get_timeouts()
        print('Standby: {}, Suspend: {}, Off: {}\n'.format(current_timeouts.standby_timeout,
                                                           current_timeouts.suspend_timeout,
                                                           current_timeouts.off_timeout))

    def toggle_dpms(self):
        current_info = self.d.dpms_info()
        if current_info.state:
            self.d.dpms_disable()
        else:
            self.d.dpms_enable()

        self.d.sync()

    def restore(self):
        print('Restoring DPMS configuration')
        self.d.dpms_set_timeouts(self.initial_timeouts.standby_timeout,
                                 self.initial_timeouts.suspend_timeout,
                                 self.initial_timeouts.off_timeout)
        if self.initial_info.state:
            self.d.dpms_enable()
        else:
            self.d.dpms_disable()

        self.d.sync()

        self.print_dpms()

    def set_random_timeouts(self):
        # Can be any number greater than 0
        # Using 10 just to not turnoff the screen suddenly
        standby_timeout = random.randint(10, 600)
        # Shouldn't be smaller than standby_timeout
        suspend_timeout = random.randint(standby_timeout, 600)
        # Shouldn't be smaller than standby_timeout or suspend_timeout
        off_timeout = random.randint(suspend_timeout, 600)
        self.d.dpms_set_timeouts(standby_timeout, suspend_timeout, off_timeout)
        self.d.sync()

    def turn_off_display(self):
        self.d.dpms_force_level(dpms.DPMSModeOff)
        self.d.sync()

    def turn_on_display(self):
        self.d.dpms_force_level(dpms.DPMSModeOn)
        self.d.sync()


def main():
    try:
        examples = DPMSExamples()

        print('Initial state')
        examples.print_dpms()

        print('Setting random timeouts')
        examples.set_random_timeouts()
        examples.print_dpms()

        print('The next example will turn-off your screen, press Ctrl-C to cancel.')
        time.sleep(2)
        examples.turn_off_display()

        print('Turning it on again...')
        time.sleep(2)
        examples.turn_on_display()

        print()

        print('Toggle DPMS')
        examples.toggle_dpms()
        examples.print_dpms()

        print('Toggle it again')
        examples.toggle_dpms()
        examples.print_dpms()
    finally:
        examples.restore()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# examples/xinerama.py -- demonstrate the Xinerama extension
#
#    Copyright (C) 2009 David H. Bronke <whitelynx@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA


# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os
import pprint

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib import X, display, Xutil
from Xlib.ext import xinerama

# Application window (only one)
class Window(object):
    def __init__(self, display):
        self.d = display

        # Check for extension
        if not self.d.has_extension('XINERAMA'):
            sys.stderr.write('{}: server does not have the XINERAMA extension\n'.format(sys.argv[0]))
            ext = self.d.query_extension('XINERAMA')
            print(ext)
            sys.stderr.write("\n".join(self.d.list_extensions()))
            if ext is None:
                sys.exit(1)

        # print version
        r = self.d.xinerama_query_version()
        print('XINERAMA version %d.%d' % (r.major_version, r.minor_version))


        # Grab the current screen
        self.screen = self.d.screen()

        self.window = self.screen.root.create_window(
            50, 50, 300, 200, 2,
            self.screen.root_depth,
            X.InputOutput,
            X.CopyFromParent,

            # special attribute values
            background_pixel = self.screen.white_pixel,
            event_mask = (X.ExposureMask |
                          X.StructureNotifyMask |
                          X.ButtonPressMask |
                          X.ButtonReleaseMask |
                          X.Button1MotionMask),
            colormap = X.CopyFromParent,
            )

        self.gc = self.window.create_gc(
            foreground = self.screen.black_pixel,
            background = self.screen.white_pixel,
            )

        # Set some WM info

        self.WM_DELETE_WINDOW = self.d.intern_atom('WM_DELETE_WINDOW')
        self.WM_PROTOCOLS = self.d.intern_atom('WM_PROTOCOLS')

        self.window.set_wm_name('Xlib example: xinerama.py')
        self.window.set_wm_icon_name('xinerama.py')
        self.window.set_wm_class('xinerama', 'XlibExample')

        self.window.set_wm_protocols([self.WM_DELETE_WINDOW])
        self.window.set_wm_hints(flags = Xutil.StateHint,
                                 initial_state = Xutil.NormalState)

        self.window.set_wm_normal_hints(flags = (Xutil.PPosition | Xutil.PSize
                                                 | Xutil.PMinSize),
                                        min_width = 20,
                                        min_height = 20)

        # Map the window, making it visible
        self.window.map()

        self.pp = pprint.PrettyPrinter(indent=4)

        print("Xinerama active:", bool(self.d.xinerama_is_active()))

        print("Screen info:")
        self.pp.pprint(self.d.xinerama_query_screens()._data)

        # FIXME: This doesn't work!
        #print("Xinerama info:")
        #self.pp.pprint(self.d.xinerama_get_info(self.d.screen().root_visual)._data)

        print("Xinerama state:")
        self.pp.pprint(self.window.xinerama_get_state()._data)

        print("Screen count:")
        self.pp.pprint(self.window.xinerama_get_screen_count()._data)

        for screennum in range(self.window.xinerama_get_screen_count().screen_count):
            print("Screen %d size:" % (screennum, ))
            self.pp.pprint(self.window.xinerama_get_screen_size(screennum)._data)

    def parseModes(self, mode_names, modes):
        lastIdx = 0
        modedatas = dict()
        for mode in modes:
            modedata = dict(mode._data)
            modedata['name'] = mode_names[lastIdx:lastIdx + modedata['name_length']]
            modedatas[modedata['id']] = modedata
            lastIdx += modedata['name_length']
        return modedatas

    # Main loop, handling events
    def loop(self):
        current = None
        while 1:
            e = self.d.next_event()

            # Window has been destroyed, quit
            if e.type == X.DestroyNotify:
                sys.exit(0)

            # Somebody wants to tell us something
            elif e.type == X.ClientMessage:
                if e.client_type == self.WM_PROTOCOLS:
                    fmt, data = e.data
                    if fmt == 32 and data[0] == self.WM_DELETE_WINDOW:
                        sys.exit(0)


if __name__ == '__main__':
    Window(display.Display()).loop()



# Distutils script for python-xlib

from pkg_resources import parse_requirements
from setuptools import (__version__ as setuptools_version, setup)


# Check setuptools is recent enough to support `setup.cfg`.
setuptools_require = next(parse_requirements('setuptools>=30.3.0'))
assert setuptools_version in setuptools_require, '{} is required'.format(setuptools_require)


setup(
    install_requires=['six>=1.10.0'],
    setup_requires=['setuptools-scm'],
    packages=[
        'Xlib',
        'Xlib.ext',
        'Xlib.keysymdef',
        'Xlib.protocol',
        'Xlib.support',
        'Xlib.xobject'
    ],
)



#!/usr/bin/env python
#
# examples/run_examples.py -- run some examples.
#
#    Copyright (C) 2016 Svetlana Filicheva
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os
import subprocess
import unittest

examples_folder = os.path.abspath(os.path.dirname(__file__)) + "/"

def run_example(path):
    """ Returns returncode of example """
    cmd = "{0} {1}".format(sys.executable, path)
    proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    res = proc.communicate()
    if proc.returncode:
        print(res[1].decode())
    return proc.returncode

class TestExamples(unittest.TestCase):
    """ Run some of examples """

# TODO
#     def test_eventthread(self):
#         """ Run eventthread.py -- Tests multithreaded event handling """
#         self.assertEqual(run_example(examples_folder + "eventthread.py"), 0)

    def test_get_selection(self):
        """ Run get_selection.py -- demonstrate getting selections """
        self.assertEqual(run_example(examples_folder + "get_selection.py PRIMARY"), 0)
        self.assertEqual(run_example(examples_folder + "get_selection.py SECONDARY"), 0)
        self.assertEqual(run_example(examples_folder + "get_selection.py CLIPBOARD"), 0)

    def test_profilex(self):
        """ Run profilex.py -- program to generate profiling data """
        self.assertEqual(run_example(examples_folder + "profilex.py " + examples_folder + "profilex_output"), 0)
        subprocess.call(["rm", examples_folder + "profilex_output"])

# TODO
#     def test_record_demo(self):
#         """ Run record_demo.py -- demonstrate record extension """
#         self.assertEqual(run_example(examples_folder + "record_demo.py"), 0)

    def test_security(self):
        """ Run security.py -- demonstrate the SECURITY extension """
        self.assertEqual(run_example(examples_folder + "security.py --generate"), 0)
        self.assertEqual(run_example(examples_folder + "security.py --revoke"), 0)

    def test_xfixes(self):
        """ Run xfixes.py -- demonstrate the XFIXES extension """
        self.assertEqual(run_example(examples_folder + "xfixes.py"), 0)

    def test_xlsatoms(self):
        """ Run xlsatoms.py -- show list atoms on X server """
        self.assertEqual(run_example(examples_folder + "xlsatoms.py"), 0)

    def test_xres(self):
        """ Run xres.py -- demonstrate the X-Resource extension """
        self.assertEqual(run_example(examples_folder + "xres.py"), 0)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# examples/eventthread.py -- Tests multithreaded event handling
#
#    Copyright (C) 2010-2011 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import time
from threading import Thread

from Xlib import Xatom, threaded
from Xlib.display import Display


class EventThread(Thread):
    display = None
    _stop = None

    def __init__(self, display):
        super(EventThread, self).__init__()
        self.display = display
        self.daemon = True

    def run(self):
        while True:
            event = self.display.next_event()
            print('event: %r' % event)


def main(argv):
    display = Display()

    thread = EventThread(display)
    thread.start()
    time.sleep(1)

    screen = display.screen()

    # The get_property call should not deadlock, despite the blocking next_event
    # call in the thread.
    atom = display.intern_atom('_XROOTPMAP_ID', True)
    response = screen.root.get_property(atom, Xatom.PIXMAP, 0, 1)
    print('get_property response: %r' % response)

    display.close()


if __name__ == '__main__':
    sys.exit(main(sys.argv))




from collections import OrderedDict
from functools import partial
import struct
import types
import re

from six import binary_type, iterbytes

from Xlib.protocol import rq
from . import DummyDisplay, TestCase


dummy_display = DummyDisplay()

def pack(format, *args):
    return struct.pack('=' + format, *args)

def packstr(s, padding=0):
    return s.encode() + (b'\0' * padding)


class StructTest(object):
    """ Test class helper for rq.Struct packing/unpacking support. """

    struct = None
    values_in = None
    binary = None
    values_out = None

    def test_pack_value_dict(self):
        self.assertBinaryEqual(self.struct.pack_value(dict(self.values_in)), self.binary)

    def test_pack_value_tuple(self):
        self.assertBinaryEqual(self.struct.pack_value(tuple(self.values_in.values())), self.binary)

    def test_to_binary_args(self):
        self.assertBinaryEqual(self.struct.to_binary(*self.values_in.values()), self.binary)

    def test_to_binary_kwargs(self):
        self.assertBinaryEqual(self.struct.to_binary(**dict(self.values_in)), self.binary)

    def test_parse_binary(self):
        values, remain = self.struct.parse_binary(self.binary, dummy_display)
        self.assertEqual(values, rq.DictWrapper(dict(self.values_out)))
        self.assertBinaryEmpty(remain)


def _struct_test(name, fields):
    """ Generate a new Struct packing/unpacking test.

    <fields> is a list of (field_name, field_type, field_value, field_binary):
        - field_name:
            name of the field, None if it's not a named field (e.g. LengthOf)
        - field_type:
            factory for the field type, taking one parameter only, the name of
            the field and returning a rq.Field subclass instance
        - field_value_in:
            the raw value passed in when packing and expected output on unpacking
        - field_binary:
            the binary representation of the field value or a function to
            convert it to binary for creating the expected result of packing
            the struct
        - field_value_out:
            the raw value passed expected on output on unpacking
            (optional, <field_value_in> is used if not specified)
    """

    class_name = ''.join([part.capitalize() for part in re.sub(r'[^\w]+', ' ', name).split()])
    class_name += 'StructTest'
    struct_layout = []
    values_in = OrderedDict()
    binary = b''
    values_out = OrderedDict()
    for field_params in fields:
        assert 4 <= len(field_params) <= 5
        field_name, field_type, field_value_in, field_binary = field_params[0:4]
        if len(field_params) == 5:
            field_value_out = field_params[4]
        else:
            field_value_out = field_value_in
        struct_layout.append(field_type(field_name))
        if field_name is not None:
            if field_value_in is not None:
                values_in[field_name] = field_value_in
            if field_value_out is not None:
                values_out[field_name] = field_value_out
        if isinstance(field_binary, binary_type):
            binary += field_binary
        elif isinstance(field_binary, (types.FunctionType, types.LambdaType, partial)):
            binary += field_binary(field_value_in)
        else:
            raise ValueError('unsupported type for binary: {data} [{type}]'.format(
                data=str(field_binary), type=type(field_binary))
            )
    class_dict = {
        'struct': rq.Struct(*struct_layout),
        'values_in': values_in,
        'binary': binary,
        'values_out': values_out,
    }
    globals()[class_name] = type(class_name, (StructTest, TestCase), class_dict)


_struct_test('card', (
    ('c8' , rq.Card8 , 0x42      , partial(pack, 'B')),
    ('c16', rq.Card16, 0x666     , partial(pack, 'H')),
    ('c32', rq.Card32, 0xdeadbeef, partial(pack, 'L')),
))

_struct_test('int', (
    ('i8' , rq.Int8 , -42        , partial(pack, 'b')),
    ('i16', rq.Int16, -666       , partial(pack, 'h')),
    ('i32', rq.Int32, -2147483648, partial(pack, 'l')),
))

_struct_test('list', (
    (None   , lambda name: rq.LengthOf('lc8', 1)          , None                                   , pack('B', 2)              ),
    (None   , lambda name: rq.LengthOf('lc16', 2)         , None                                   , pack('H', 3)              ),
    (None   , lambda name: rq.LengthOf('lc32', 4)         , None                                   , pack('L', 5)              ),
    ('lc8'  , lambda name: rq.List(name, rq.Card8 , pad=0), [0x42, 0xc3]                           , lambda v: pack('2B'  , *v)),
    ('lc16' , lambda name: rq.List(name, rq.Card16, pad=0), [666, 1, 60143]                        , lambda v: pack('3H'  , *v)),
    ('lc32' , lambda name: rq.List(name, rq.Card32, pad=0), [0xf02facb, 666, 1, 0x1043, 0xdeadbeef], lambda v: pack('5L'  , *v)),
))

_struct_test('list pad', (
    (None   , lambda name: rq.LengthOf('lc8', 1)          , None                                   , pack('B', 2)              ),
    (None   , lambda name: rq.LengthOf('lc16', 2)         , None                                   , pack('H', 3)              ),
    (None   , lambda name: rq.LengthOf('lc32', 4)         , None                                   , pack('L', 5)              ),
    ('lc8'  , lambda name: rq.List(name, rq.Card8 , pad=1), [0x42, 0xc3]                           , lambda v: pack('2B2x', *v)),
    ('lc16' , lambda name: rq.List(name, rq.Card16, pad=1), [666, 1, 60143]                        , lambda v: pack('3H2x', *v)),
    ('lc32' , lambda name: rq.List(name, rq.Card32, pad=1), [0xf02facb, 666, 1, 0x1043, 0xdeadbeef], lambda v: pack('5L'  , *v)),
))

_struct_test('fixed list', (
    ('lc8'  , lambda name: rq.FixedList(name, 2, rq.Card8 , pad=0), [0x42, 0xc3]                           , lambda v: pack('2B'  , *v)),
    ('lc16' , lambda name: rq.FixedList(name, 3, rq.Card16, pad=0), [666, 1, 60143]                        , lambda v: pack('3H'  , *v)),
    ('lc32' , lambda name: rq.FixedList(name, 5, rq.Card32, pad=0), [0xf02facb, 666, 1, 0x1043, 0xdeadbeef], lambda v: pack('5L'  , *v)),
))

_struct_test('fixed list pad', (
    ('lc8p' , lambda name: rq.FixedList(name, 2, rq.Card8 , pad=1), [0x42, 0xc3]                           , lambda v: pack('2B2x', *v)),
    ('lc16p', lambda name: rq.FixedList(name, 3, rq.Card16, pad=1), [666, 1, 60143]                        , lambda v: pack('3H2x', *v)),
    ('lc32p', lambda name: rq.FixedList(name, 5, rq.Card32, pad=1), [0xf02facb, 666, 1, 0x1043, 0xdeadbeef], lambda v: pack('5L'  , *v)),
))

_struct_test('string8', (
    (None, lambda name: rq.LengthOf('s1', 1)   , None                                , pack('B', 7) ),
    (None, lambda name: rq.LengthOf('s2', 2)   , None                                , pack('H', 13)),
    (None, lambda name: rq.LengthOf('s3', 4)   , None                                , pack('L', 34)),
    ('s1', lambda name: rq.String8(name, pad=0), "testing"                           , packstr      ),
    ('s2', lambda name: rq.String8(name, pad=0), "one two three"                     , packstr      ),
    ('s3', lambda name: rq.String8(name, pad=0), "supercalifragilisticexpialidocious", packstr      ),
))

_struct_test('string8 pad', (
    (None, lambda name: rq.LengthOf('s1', 1)   , None                                , pack('B', 7)               ),
    (None, lambda name: rq.LengthOf('s2', 2)   , None                                , pack('H', 13)              ),
    (None, lambda name: rq.LengthOf('s3', 4)   , None                                , pack('L', 34)              ),
    ('s1', lambda name: rq.String8(name, pad=1), "testing"                           , partial(packstr, padding=1)),
    ('s2', lambda name: rq.String8(name, pad=1), "one two three"                     , partial(packstr, padding=3)),
    ('s3', lambda name: rq.String8(name, pad=1), "supercalifragilisticexpialidocious", partial(packstr, padding=2)),
))

_struct_test('simple string16', (
    (None, lambda name: rq.LengthOf('s1', 1)   , None            , pack('B', 3) ),
    (None, lambda name: rq.LengthOf('s2', 2)   , None            , pack('H', 3) ),
    ('s1', lambda name: rq.String16(name, pad=0), (0, 1, 2), lambda s: struct.pack('>' + 'H' * len(s), *s)),
    # An 8-bits string is also allowed on input.
    ('s2', lambda name: rq.String16(name, pad=0), b'\x03\x04\x05', lambda s: struct.pack('>' + 'H' * len(s), *iterbytes(s)), (3, 4, 5)),
))

_struct_test('binary', (
    (None, lambda name: rq.LengthOf('s1', 1)  , None                                 , pack('B', 7) ),
    (None, lambda name: rq.LengthOf('s2', 2)  , None                                 , pack('H', 13)),
    (None, lambda name: rq.LengthOf('s3', 4)  , None                                 , pack('L', 34)),
    ('s1', lambda name: rq.Binary(name, pad=0), b"testing"                           , lambda v: v),
    ('s2', lambda name: rq.Binary(name, pad=0), b"one two three"                     , lambda v: v),
    ('s3', lambda name: rq.Binary(name, pad=0), b"supercalifragilisticexpialidocious", lambda v: v),
))

_struct_test('binary pad', (
    (None, lambda name: rq.LengthOf('s1', 1)  , None                                 , pack('B', 7) ),
    (None, lambda name: rq.LengthOf('s2', 2)  , None                                 , pack('H', 13)),
    (None, lambda name: rq.LengthOf('s3', 4)  , None                                 , pack('L', 34)),
    ('s1', lambda name: rq.Binary(name, pad=1), b"testing"                           , lambda v: v + b'\0' * 1),
    ('s2', lambda name: rq.Binary(name, pad=1), b"one two three"                     , lambda v: v + b'\0' * 3),
    ('s3', lambda name: rq.Binary(name, pad=1), b"supercalifragilisticexpialidocious", lambda v: v + b'\0' * 2),
))

_struct_test('fixed binary', (
    ('s1', lambda name: rq.FixedBinary(name, 7 ), b"testing"                           , lambda v: v),
    ('s2', lambda name: rq.FixedBinary(name, 13), b"one two three"                     , lambda v: v),
    ('s3', lambda name: rq.FixedBinary(name, 34), b"supercalifragilisticexpialidocious", lambda v: v),
    # Make sure fixed binary fields are handled as static fields.
    ('c8', rq.Card8, 0x42, partial(pack, 'B')),
))



#!/usr/bin/env python2

import sys, os
sys.path.insert(0, os.path.normpath(os.path.join(__file__, '../..')))

import unittest
from Xlib.protocol import request, event
from . import BigEndianTest as EndianTest
from . import DummyDisplay

dummy_display = DummyDisplay()


class TestKeymapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'data': [248, 202, 136, 187, 201, 200, 244, 146, 187, 195, 178, 236, 157, 185, 166, 196, 164, 146, 143, 170, 206, 201, 240, 159, 247, 205, 231, 197, 254, 240, 148],
            'type': 138,
            }
        self.evt_bin_0 = b'\x8a\xf8\xca\x88' b'\xbb\xc9\xc8\xf4' \
            b'\x92\xbb\xc3\xb2' b'\xec\x9d\xb9\xa6' \
            b'\xc4\xa4\x92\x8f' b'\xaa\xce\xc9\xf0' \
            b'\x9f\xf7\xcd\xe7' b'\xc5\xfe\xf0\x94'


    def testPack0(self):
        bin = event.KeymapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.KeymapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 15258,
            'height': 6241,
            'sequence_number': 48394,
            'type': 238,
            'width': 1951,
            'window': 692245859,
            'x': 39820,
            'y': 16664,
            }
        self.evt_bin_0 = b'\xee\x00\xbd\x0a' b'\x29\x42\xd5\x63' \
            b'\x9b\x8c\x41\x18' b'\x07\x9f\x18\x61' \
            b'\x3b\x9a\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.Expose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.Expose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestGraphicsExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 49818,
            'drawable': 1443779242,
            'height': 2892,
            'major_event': 172,
            'minor_event': 50267,
            'sequence_number': 50375,
            'type': 133,
            'width': 38020,
            'x': 54088,
            'y': 17918,
            }
        self.evt_bin_0 = b'\x85\x00\xc4\xc7' b'\x56\x0e\x52\xaa' \
            b'\xd3\x48\x45\xfe' b'\x94\x84\x0b\x4c' \
            b'\xc4\x5b\xc2\x9a' b'\xac\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.GraphicsExpose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.GraphicsExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestNoExpose(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'major_event': 199,
            'minor_event': 29237,
            'sequence_number': 34266,
            'type': 248,
            'window': 1399171519,
            }
        self.evt_bin_0 = b'\xf8\x00\x85\xda' b'\x53\x65\xa9\xbf' \
            b'\x72\x35\xc7\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.NoExpose._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.NoExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestVisibilityNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'sequence_number': 38616,
            'state': 253,
            'type': 174,
            'window': 936121409,
            }
        self.evt_bin_0 = b'\xae\x00\x96\xd8' b'\x37\xcc\x14\x41' \
            b'\xfd\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.VisibilityNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.VisibilityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestCreateNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'border_width': 56468,
            'height': 7111,
            'override': 0,
            'parent': 747306217,
            'sequence_number': 31058,
            'type': 151,
            'width': 44173,
            'window': 876986399,
            'x': -21847,
            'y': -22248,
            }
        self.evt_bin_0 = b'\x97\x00\x79\x52' b'\x2c\x8a\xfc\xe9' \
            b'\x34\x45\xc0\x1f' b'\xaa\xa9\xa9\x18' \
            b'\xac\x8d\x1b\xc7' b'\xdc\x94\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.CreateNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.CreateNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestDestroyNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1489718405,
            'sequence_number': 27233,
            'type': 212,
            'window': 1064077163,
            }
        self.evt_bin_0 = b'\xd4\x00\x6a\x61' b'\x58\xcb\x4c\x85' \
            b'\x3f\x6c\x87\x6b' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.DestroyNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.DestroyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestUnmapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 2029853215,
            'from_configure': 0,
            'sequence_number': 43679,
            'type': 201,
            'window': 860030193,
            }
        self.evt_bin_0 = b'\xc9\x00\xaa\x9f' b'\x78\xfd\x1a\x1f' \
            b'\x33\x43\x04\xf1' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.UnmapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.UnmapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestMapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 675485985,
            'override': 1,
            'sequence_number': 6027,
            'type': 244,
            'window': 542087937,
            }
        self.evt_bin_0 = b'\xf4\x00\x17\x8b' b'\x28\x43\x19\x21' \
            b'\x20\x4f\x9b\x01' b'\x01\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestMapRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'parent': 1659099581,
            'sequence_number': 63838,
            'type': 157,
            'window': 868024861,
            }
        self.evt_bin_0 = b'\x9d\x00\xf9\x5e' b'\x62\xe3\xd9\xbd' \
            b'\x33\xbd\x02\x1d' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MapRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MapRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestReparentNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1867017989,
            'override': 0,
            'parent': 992152190,
            'sequence_number': 43356,
            'type': 128,
            'window': 1165276406,
            'x': -19227,
            'y': -30992,
            }
        self.evt_bin_0 = b'\x80\x00\xa9\x5c' b'\x6f\x48\x6f\x05' \
            b'\x45\x74\xb4\xf6' b'\x3b\x23\x0a\x7e' \
            b'\xb4\xe5\x86\xf0' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ReparentNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ReparentNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestConfigureNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'above_sibling': 1343120585,
            'border_width': 17757,
            'event': 1624514845,
            'height': 13596,
            'override': 0,
            'sequence_number': 41060,
            'type': 220,
            'width': 3638,
            'window': 1070571314,
            'x': -18284,
            'y': -7865,
            }
        self.evt_bin_0 = b'\xdc\x00\xa0\x64' b'\x60\xd4\x21\x1d' \
            b'\x3f\xcf\x9f\x32' b'\x50\x0e\x64\xc9' \
            b'\xb8\x94\xe1\x47' b'\x0e\x36\x35\x1c' \
            b'\x45\x5d\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ConfigureNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ConfigureNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestConfigureRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'border_width': 52639,
            'height': 40159,
            'parent': 1499546058,
            'sequence_number': 57983,
            'sibling': 260826075,
            'stack_mode': 240,
            'type': 201,
            'value_mask': 15938,
            'width': 41545,
            'window': 1040976198,
            'x': -31823,
            'y': -880,
            }
        self.evt_bin_0 = b'\xc9\xf0\xe2\x7f' b'\x59\x61\x41\xca' \
            b'\x3e\x0c\x09\x46' b'\x0f\x8b\xe3\xdb' \
            b'\x83\xb1\xfc\x90' b'\xa2\x49\x9c\xdf' \
            b'\xcd\x9f\x3e\x42' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ConfigureRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ConfigureRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestGravityNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'event': 1182983311,
            'sequence_number': 20621,
            'type': 168,
            'window': 29431224,
            'x': -14672,
            'y': -19399,
            }
        self.evt_bin_0 = b'\xa8\x00\x50\x8d' b'\x46\x82\xe4\x8f' \
            b'\x01\xc1\x15\xb8' b'\xc6\xb0\xb4\x39' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.GravityNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.GravityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestResizeRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'height': 60085,
            'sequence_number': 14981,
            'type': 151,
            'width': 55398,
            'window': 2130921516,
            }
        self.evt_bin_0 = b'\x97\x00\x3a\x85' b'\x7f\x03\x48\x2c' \
            b'\xd8\x66\xea\xb5' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ResizeRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ResizeRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestPropertyNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'atom': 252322161,
            'sequence_number': 30497,
            'state': 167,
            'time': 1391011497,
            'type': 157,
            'window': 2033863003,
            }
        self.evt_bin_0 = b'\x9d\x00\x77\x21' b'\x79\x3a\x49\x5b' \
            b'\x0f\x0a\x21\x71' b'\x52\xe9\x26\xa9' \
            b'\xa7\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.PropertyNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.PropertyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionClear(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'atom': 1092859866,
            'sequence_number': 61378,
            'time': 1538959461,
            'type': 255,
            'window': 626833463,
            }
        self.evt_bin_0 = b'\xff\x00\xef\xc2' b'\x5b\xba\xa8\x65' \
            b'\x25\x5c\xb8\x37' b'\x41\x23\xb7\xda' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionClear._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionClear._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionRequest(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'owner': 1036225485,
            'property': 981007010,
            'requestor': 2105124856,
            'selection': 1014149797,
            'sequence_number': 27485,
            'target': 523473665,
            'time': 1792621552,
            'type': 197,
            }
        self.evt_bin_0 = b'\xc5\x00\x6b\x5d' b'\x6a\xd9\x3b\xf0' \
            b'\x3d\xc3\x8b\xcd' b'\x7d\x79\xa7\xf8' \
            b'\x3c\x72\xb2\xa5' b'\x1f\x33\x93\x01' \
            b'\x3a\x78\xfa\xa2' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionRequest._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestSelectionNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'property': 1602716574,
            'requestor': 1979762314,
            'selection': 1287219120,
            'sequence_number': 25394,
            'target': 1091504539,
            'time': 409398186,
            'type': 165,
            }
        self.evt_bin_0 = b'\xa5\x00\x63\x32' b'\x18\x66\xeb\xaa' \
            b'\x76\x00\xc6\x8a' b'\x4c\xb9\x67\xb0' \
            b'\x41\x0f\x09\x9b' b'\x5f\x87\x83\x9e' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.SelectionNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.SelectionNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestColormapNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'colormap': 593302316,
            'new': 1,
            'sequence_number': 56880,
            'state': 215,
            'type': 162,
            'window': 149981547,
            }
        self.evt_bin_0 = b'\xa2\x00\xde\x30' b'\x08\xf0\x89\x6b' \
            b'\x23\x5d\x13\x2c' b'\x01\xd7\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.ColormapNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ColormapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


class TestClientMessage(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'client_type': 607457628,
            'data': (8, b'01234567890123456789'),
            'sequence_number': 54031,
            'type': 196,
            'window': 5574388,
            }
        self.evt_bin_0 = b'\xc4\x08\xd3\x0f' b'\x00\x55\x0e\xf4' \
            b'\x24\x35\x11\x5c' b'\x30\x31\x32\x33' \
            b'\x34\x35\x36\x37' b'\x38\x39\x30\x31' \
            b'\x32\x33\x34\x35' b'\x36\x37\x38\x39'

        self.evt_args_1 = {
            'client_type': 1245441508,
            'data': (16, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
            'sequence_number': 55140,
            'type': 204,
            'window': 1260504694,
            }
        self.evt_bin_1 = b'\xcc\x10\xd7\x64' b'\x4b\x21\xc6\x76' \
            b'\x4a\x3b\xed\xe4' b'\x00\x01\x00\x02' \
            b'\x00\x03\x00\x04' b'\x00\x05\x00\x06' \
            b'\x00\x07\x00\x08' b'\x00\x09\x00\x0a'

        self.evt_args_2 = {
            'client_type': 959018764,
            'data': (32, [1, 2, 3, 4, 5]),
            'sequence_number': 56961,
            'type': 253,
            'window': 319171761,
            }
        self.evt_bin_2 = b'\xfd\x20\xde\x81' b'\x13\x06\x2c\xb1' \
            b'\x39\x29\x77\x0c' b'\x00\x00\x00\x01' \
            b'\x00\x00\x00\x02' b'\x00\x00\x00\x03' \
            b'\x00\x00\x00\x04' b'\x00\x00\x00\x05'


    def testPack0(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)

    def testPack1(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_1)
        self.assertBinaryEqual(bin, self.evt_bin_1)

    def testUnpack1(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_1, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_1)

    def testPack2(self):
        bin = event.ClientMessage._fields.to_binary(*(), **self.evt_args_2)
        self.assertBinaryEqual(bin, self.evt_bin_2)

    def testUnpack2(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_2, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_2)


class TestMappingNotify(EndianTest):
    def setUp(self):
        self.evt_args_0 = {
            'count': 244,
            'first_keycode': 224,
            'request': 213,
            'sequence_number': 22874,
            'type': 251,
            }
        self.evt_bin_0 = b'\xfb\x00\x59\x5a' b'\xd5\xe0\xf4\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00' \
            b'\x00\x00\x00\x00' b'\x00\x00\x00\x00'


    def testPack0(self):
        bin = event.MappingNotify._fields.to_binary(*(), **self.evt_args_0)
        self.assertBinaryEqual(bin, self.evt_bin_0)

    def testUnpack0(self):
        args, remain = event.MappingNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        self.assertBinaryEmpty(remain)
        self.assertEqual(args, self.evt_args_0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
#
# examples/security.py -- demonstrate the SECURITY extension
#
#    Copyright (C) 2011 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# Python 2/3 compatibility.
from __future__ import print_function

import sys, os
from optparse import OptionParser

# Change path so we find Xlib
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from Xlib.display import Display
from Xlib.ext import security


def main(argv):
    parser = OptionParser()

    parser.add_option('--generate', action='store_true', default=False)
    parser.add_option('--proto', default='MIT-MAGIC-COOKIE-1')
    parser.add_option('--trusted', action='store_true', default=False)
    parser.add_option('--untrusted', action='store_true', default=False)

    parser.add_option('--revoke', action='store_true', default=False)

    opts, args = parser.parse_args(argv[1:])

    if opts.trusted and opts.untrusted:
        parser.error('--trusted and --untrusted cannot be combined')

    if not any((opts.generate, opts.revoke)):
        parser.error('specify --generate or --revoke')

    display = Display()

    if not display.has_extension('SECURITY'):
        if display.query_extension('SECURITY') is None:
            print('SECURITY extension not supported', file=sys.stderr)
            return 1

    security_version = display.security_query_version()
    print('SECURITY version %s.%s' % (
      security_version.major_version,
      security_version.minor_version,
    ), file=sys.stderr)

    if opts.generate:
        kwargs = {}
        if opts.trusted:
            kwargs['trust_level'] = security.SecurityClientTrusted
        elif opts.untrusted:
            kwargs['trust_level'] = security.SecurityClientUntrusted
        reply = display.security_generate_authorization(opts.proto, **kwargs)
        print(reply.authid)

    elif opts.revoke:
        for arg in args:
            authid = int(arg, 10)
            display.security_revoke_authorization(authid)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/python
#
# examples/nvcontrol.py -- demonstrate the NV-CONTROL extension
#
#    Copyright (C) 2019 Roberto Leinardi <roberto@leinardi.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA


# Python 2/3 compatibility.
from __future__ import print_function

import sys
import os

# Change path so we find Xlib
from pprint import pprint

from Xlib.display import Display
from Xlib.ext.nvcontrol import Gpu, Cooler

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

if __name__ == '__main__':
    display = Display()
    # Check for extension
    if not display.has_extension('NV-CONTROL'):
        sys.stderr.write('{}: server does not have the NV-CONTROL extension\n'.format(sys.argv[0]))
        ext = display.query_extension('NV-CONTROL')
        print(ext)
        sys.stderr.write("\n".join(display.list_extensions()))
        if ext is None:
            sys.exit(1)

    gpu = Gpu(0)
    fan = Cooler(0)

    perf_level = 3

    dic = {
        'get_gpu_count': display.nvcontrol_get_gpu_count(),
        'get_vram': display.nvcontrol_get_vram(gpu),
        'get_irq': display.nvcontrol_get_irq(gpu),
        'supports_framelock': display.nvcontrol_supports_framelock(gpu),
        'get_core_temp': display.nvcontrol_get_core_temp(gpu),
        'get_core_threshold': display.nvcontrol_get_core_threshold(gpu),
        'get_default_core_threshold': display.nvcontrol_get_default_core_threshold(gpu),
        'get_max_core_threshold': display.nvcontrol_get_max_core_threshold(gpu),
        'get_ambient_temp': display.nvcontrol_get_ambient_temp(gpu),
        'get_cuda_cores': display.nvcontrol_get_cuda_cores(gpu),
        'get_memory_bus_width': display.nvcontrol_get_memory_bus_width(gpu),
        'get_total_dedicated_gpu_memory': display.nvcontrol_get_total_dedicated_gpu_memory(gpu),
        'get_used_dedicated_gpu_memory': display.nvcontrol_get_used_dedicated_gpu_memory(gpu),
        'get_curr_pcie_link_width': display.nvcontrol_get_curr_pcie_link_width(gpu),
        'get_max_pcie_link_width': display.nvcontrol_get_max_pcie_link_width(gpu),
        'get_curr_pcie_link_generation': display.nvcontrol_get_curr_pcie_link_generation(gpu),
        'get_encoder_utilization': display.nvcontrol_get_encoder_utilization(gpu),
        'get_decoder_utilization': display.nvcontrol_get_decoder_utilization(gpu),
        'get_current_performance_level': display.nvcontrol_get_current_performance_level(gpu),
        'get_gpu_nvclock_offset': display.nvcontrol_get_gpu_nvclock_offset(gpu, perf_level),
        'get_mem_transfer_rate_offset': display.nvcontrol_get_mem_transfer_rate_offset(gpu, perf_level),
        'get_cooler_manual_control_enabled': display.nvcontrol_get_cooler_manual_control_enabled(gpu),
        'get_fan_duty': display.nvcontrol_get_fan_duty(fan),
        'get_fan_rpm': display.nvcontrol_get_fan_rpm(fan),
        'get_coolers_used_by_gpu': display.nvcontrol_get_coolers_used_by_gpu(gpu),
        'get_max_displays': display.nvcontrol_get_max_displays(gpu),
        'get_name': display.nvcontrol_get_name(gpu),
        'get_driver_version': display.nvcontrol_get_driver_version(gpu),
        'get_vbios_version': display.nvcontrol_get_vbios_version(gpu),
        'get_gpu_uuid': display.nvcontrol_get_gpu_uuid(gpu),
        'get_utilization_rates': display.nvcontrol_get_utilization_rates(gpu),
        'get_performance_modes': display.nvcontrol_get_performance_modes(gpu),
        'get_gpu_nvclock_offset_range': display.nvcontrol_get_gpu_nvclock_offset_range(gpu, perf_level),
        'get_mem_transfer_rate_offset_range': display.nvcontrol_get_mem_transfer_rate_offset_range(gpu, perf_level),
        'get_clock_info': display.nvcontrol_get_clock_info(gpu)
    }

    pprint(dic)

    display.close()
